@@ -3,19 +3,8 @@
 """
 Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from qt import Qt, QPointF, QPoint, QRectF, QMimeData, QSize
@@ -37,11 +26,11 @@ from util import stack, uniqueList
 from log import logDebug, logException
 from mi18n import i18n, i18nc
 from common import LIGHTSOURCES, Internal, Debug, isAlive, StrMixin
-from common import ZValues
+from common import DrawOnTopMixin
 from wind import Wind, East
 
 
-class PlayerWind(AnimatedMixin, QGraphicsObject, StrMixin):
+class PlayerWind(AnimatedMixin, QGraphicsObject, StrMixin, DrawOnTopMixin):
 
     """a round wind tile"""
 
@@ -50,7 +39,7 @@ class PlayerWind(AnimatedMixin, QGraphicsObject, StrMixin):
 
     def __init__(self, wind, parent=None):
         """generate new wind marker"""
-        super(PlayerWind, self).__init__()
+        super().__init__()
         assert not parent
         assert isinstance(wind, Wind), 'wind {}  must be a real Wind but is {}'.format(
             wind, type(wind))
@@ -72,14 +61,6 @@ class PlayerWind(AnimatedMixin, QGraphicsObject, StrMixin):
         scenePos = self.board.mapToScene(boardPos)
         return {'pos': scenePos, 'rotation': sceneRotation(self.board)}
 
-    def setDrawingOrder(self):
-        """we want the winds above all others"""
-        if self.activeAnimation.get('pos'):
-            movingZ = ZValues.movingZ
-        else:
-            movingZ = 0
-        self.setZValue(ZValues.markerZ + movingZ)
-
     @property
     def wind(self):
         """our wind"""
@@ -98,7 +79,7 @@ class PlayerWind(AnimatedMixin, QGraphicsObject, StrMixin):
             newPrevailing = self.wind == Wind.all4[value % 4]
         self.__brush = self.roundWindColor if newPrevailing else self.whiteColor
 
-    def paint(self, painter, dummyOption, dummyWidget=None):
+    def paint(self, painter, unusedOption, unusedWidget=None):
         """paint the marker"""
         with Painter(painter):
             painter.setBrush(self.__brush)
@@ -110,7 +91,7 @@ class PlayerWind(AnimatedMixin, QGraphicsObject, StrMixin):
             painter.scale(0.60, 0.60)
             renderer.render(painter, self.wind.markerSvgName, self.boundingRect())
 
-    def boundingRect(self): # pylint: disable=no-self-use
+    def boundingRect(self):  # pylint: disable=no-self-use
         """define the part of the tile we want to see"""
         size = int(Internal.scene.windTileset.faceSize.height() * 1.1)
         return QRectF(QPoint(), QPoint(size, size))
@@ -178,6 +159,7 @@ class WindLabel(QLabel):
                     child.paint(painter, QStyleOptionGraphicsItem())
         return pMap
 
+
 class Board(QGraphicsRectItem, StrMixin):
 
     """ a board with any number of positioned tiles"""
@@ -239,8 +221,7 @@ class Board(QGraphicsRectItem, StrMixin):
         if focusCandidates:
             firstCandidate = focusCandidates[0]
             if self._focusTile not in focusCandidates:
-                focusCandidates = list(
-                    x for x in focusCandidates if x.sortKey() >= self.__prevPos)
+                focusCandidates = [x for x in focusCandidates if x.sortKey() >= self.__prevPos]
                 focusCandidates.append(firstCandidate)
                 self.focusTile = focusCandidates[0]
 
@@ -270,7 +251,7 @@ class Board(QGraphicsRectItem, StrMixin):
         if self._focusTile and self._focusTile.tile in Debug.focusable:
             logDebug('%s: new focus uiTile %s from %s' % (
                 self.name, self._focusTile.tile if self._focusTile else 'None', stack('')[-1]))
-        if self.hasFocus:
+        if self.hasLogicalFocus:
             self.scene().focusBoard = self
 
     def setEnabled(self, enabled):
@@ -279,23 +260,26 @@ class Board(QGraphicsRectItem, StrMixin):
         QGraphicsRectItem.setEnabled(self, enabled)
 
     def _focusableTiles(self, sortDir=Qt.Key_Right):
-        """returns a list of all tiles in this board sorted such that
+        """return a list of all tiles in this board sorted such that
         moving in the sortDir direction corresponds to going to
         the next list element.
         respect board orientation: Right Arrow should always move right
         relative to the screen, not relative to the board"""
-        return sorted([x for x in self.uiTiles if x.focusable], key=lambda x: x.sortKey(sortDir))
+        return sorted((x for x in self.uiTiles if x.focusable), key=lambda x: x.sortKey(sortDir))
 
     @property
-    def hasFocus(self):
+    def hasLogicalFocus(self):
         """defines if this board should show a focusRect
         if another board has focus, setting this to False does
-        not change scene.focusBoard"""
+        not change scene.focusBoard
+
+        Up to May 2021, this was called hasFocus, overriding QGraphicsItem.hasFocus
+        but pylint did not like that."""
         return self.scene() and self.scene().focusBoard == self and self._focusTile
 
-    @hasFocus.setter
-    def hasFocus(self, value):
-        """sets focus on this board"""
+    @hasLogicalFocus.setter
+    def hasLogicalFocus(self, value):
+        """set focus on this board"""
         if isAlive(self):
             scene = self.scene()
             if isAlive(scene):
@@ -308,7 +292,7 @@ class Board(QGraphicsRectItem, StrMixin):
 
     @staticmethod
     def mapChar2Arrow(event):
-        """maps the keys hjkl to arrows like in vi and konqueror"""
+        """map the keys hjkl to arrows like in vi and konqueror"""
         key = event.key()
         if key in Board.arrows:
             return key
@@ -336,9 +320,9 @@ class Board(QGraphicsRectItem, StrMixin):
             # has no focusable tiles. Like after declaring
             # Original Call.
             oldPos = self.focusTile.xoffset, self.focusTile.yoffset
-            tiles = list(
+            tiles = [
                 x for x in tiles if (x.xoffset,
-                                     x.yoffset) != oldPos or x == self.focusTile)
+                                     x.yoffset) != oldPos or x == self.focusTile]
             assert tiles, [str(x) for x in self.uiTiles]
             tiles.append(tiles[0])
             self.focusTile = tiles[tiles.index(self.focusTile) + 1]
@@ -349,13 +333,13 @@ class Board(QGraphicsRectItem, StrMixin):
         return uiTile
 
     def uiMeldWithTile(self, uiTile):  # pylint: disable=no-self-use
-        """returns the UI Meld with uiTile. A Board does not know about melds,
+        """return the UI Meld with uiTile. A Board does not know about melds,
         so default is to return a Meld with only uiTile"""
         return UIMeld(uiTile)
 
-    def meldVariants(self, uiTile, lowerHalf):  # pylint: disable=no-self-use,unused-argument
+    def meldVariants(self, tile, lowerHalf):  # pylint: disable=no-self-use,unused-argument
         """all possible melds that could be meant by dragging/dropping uiTile"""
-        return [Meld(uiTile)]
+        return [Meld(tile)]
 
     def chooseVariant(self, uiTile, lowerHalf=False):
         """make the user choose from a list of possible melds for the target.
@@ -380,11 +364,11 @@ class Board(QGraphicsRectItem, StrMixin):
             idx = action.data()
         return variants[idx]
 
-    def dragEnterEvent(self, dummyEvent):
+    def dragEnterEvent(self, unusedEvent):
         """drag enters the HandBoard: highlight it"""
         self.setPen(QPen(QColor(self.penColor)))
 
-    def dragLeaveEvent(self, dummyEvent):
+    def dragLeaveEvent(self, unusedEvent):
         """drag leaves the HandBoard"""
         self._noPen()
 
@@ -400,10 +384,11 @@ class Board(QGraphicsRectItem, StrMixin):
         for uiTile in self.uiTiles:
             if (uiTile.xoffset, uiTile.yoffset, uiTile.level) == (xoffset, yoffset, level):
                 return uiTile
+        return None
 
     def tilesByElement(self, element):
-        """returns all child items holding a uiTile for element"""
-        return list(x for x in self.uiTiles if x.tile is element)
+        """return all child items holding a uiTile for element"""
+        return [x for x in self.uiTiles if x.tile is element]
 
     def rotatedLightSource(self):
         """the light source we need for the original uiTile before it is rotated"""
@@ -411,9 +396,11 @@ class Board(QGraphicsRectItem, StrMixin):
         lightSourceIndex = (lightSourceIndex + sceneRotation(self) // 90) % 4
         return LIGHTSOURCES[lightSourceIndex]
 
-    def tileFacePos(self):
+    def tileFacePos(self, showShadows=None):
         """the face pos of a uiTile relative to its origin"""
-        if not Internal.Preferences.showShadows:
+        if showShadows is None:
+            showShadows = Internal.Preferences.showShadows
+        if not showShadows:
             return QPointF()
         lightSource = self.rotatedLightSource()
         xoffset = self.tileset.shadowWidth() - 1 if 'E' in lightSource else 0
@@ -425,7 +412,7 @@ class Board(QGraphicsRectItem, StrMixin):
         return QRectF(self.tileFacePos(), self.tileset.faceSize)
 
     def setPos(self, xWidth=0, xHeight=0, yWidth=0, yHeight=0):
-        """sets the position in the parent item expressing the position in tile face units.
+        """set the position in the parent item expressing the position in tile face units.
         The X position is xWidth*facewidth + xHeight*faceheight, analog for Y"""
         self.__xWidth = xWidth
         self.__xHeight = xHeight
@@ -481,7 +468,7 @@ class Board(QGraphicsRectItem, StrMixin):
         newY = self.__yWidth * width + self.__yHeight * height + offsets[1]
         QGraphicsRectItem.setPos(self, newX, newY)
 
-    def showShadowsChanged(self, dummyOldValue, newValue):
+    def showShadowsChanged(self, unusedOldValue, newValue):
         """set active lightSource"""
         for uiTile in self.uiTiles:
             uiTile.setClippingFlags()
@@ -533,7 +520,7 @@ class Board(QGraphicsRectItem, StrMixin):
                 self.placeTile(uiTile)
                 uiTile.update()
             self.computeRect()
-            if self.hasFocus:
+            if self.hasLogicalFocus:
                 self.scene().focusBoard = self
 
     def focusRectWidth(self):  # pylint: disable=no-self-use
@@ -636,15 +623,13 @@ class SelectorBoard(CourtBoard):
 
     def checkTiles(self):
         """does not apply"""
-        pass
 
     def load(self, game):
         """load the tiles according to game.ruleset"""
         for uiTile in self.uiTiles:
             uiTile.setBoard(None)
         self.uiTiles = []
-        self.allSelectorTiles = list(UITile(x)
-                                     for x in elements.all(game.ruleset))
+        self.allSelectorTiles = [UITile(x) for x in elements.all(game.ruleset)]
         self.refill()
 
     def refill(self):
@@ -687,7 +672,7 @@ class SelectorBoard(CourtBoard):
             self.__placeAvailable(myTile)
             myTile.focusable = True
         senderHand.deselect(uiMeld)
-        (senderHand if senderHand.uiTiles else self).hasFocus = True
+        (senderHand if senderHand.uiTiles else self).hasLogicalFocus = True
         self._noPen()
         animate()
 
@@ -696,9 +681,9 @@ class SelectorBoard(CourtBoard):
         assert isinstance(uiTile, UITile), uiTile
         result = UIMeld(uiTile)
         for tile in meld[1:]:
-            baseTiles = list(
+            baseTiles = [
                 x for x in self.tilesByElement(
-                    tile.exposed) if x not in result)
+                    tile.exposed) if x not in result]
             result.append(baseTiles[0])
         return result
 
@@ -721,11 +706,11 @@ class SelectorBoard(CourtBoard):
         uiTile.dark = False
         uiTile.setBoard(self, column, row)
 
-    def meldVariants(self, uiTile, lowerHalf):
-        """returns a list of possible variants based on meld. Those are logical melds."""
+    def meldVariants(self, tile, lowerHalf):
+        """return a list of possible variants based on meld. Those are logical melds."""
         # pylint: disable=too-many-locals
-        assert isinstance(uiTile, UITile)
-        wantedTile = uiTile.tile
+        assert isinstance(tile, UITile)
+        wantedTile = tile.tile
         for selectorTile in self.uiTiles:
             selectorTile.tile = selectorTile.tile.exposed
         lowerName = wantedTile.exposed
@@ -795,7 +780,7 @@ class FittingView(QGraphicsView):
             return
         Internal.mainWindow.changeAngle()
 
-    def resizeEvent(self, dummyEvent):
+    def resizeEvent(self, unusedEvent):
         """scale the scene and its background for new view size"""
         Internal.Preferences.callTrigger(
             'tilesetName')  # this redraws and resizes
@@ -841,7 +826,7 @@ class FittingView(QGraphicsView):
             uiTile = board.mapMouseTile(tiles[0])
             if uiTile.focusable:
                 board.focusTile = uiTile
-                board.hasFocus = True
+                board.hasLogicalFocus = True
                 if hasattr(Internal.scene, 'clientDialog'):
                     if Internal.scene.clientDialog:
                         Internal.scene.clientDialog.buttons[0].setFocus()
@@ -867,11 +852,11 @@ class FittingView(QGraphicsView):
                 self.dragObject = self.drag(tilePressed)
                 self.dragObject.exec_(Qt.MoveAction)
                 self.dragObject = None
-                return
+                return None
         return QGraphicsView.mouseMoveEvent(self, event)
 
     def drag(self, uiTile):
-        """returns a drag object"""
+        """return a drag object"""
         drag = QDrag(self)
         mimeData = MimeData(uiTile)
         drag.setMimeData(mimeData)
@@ -918,7 +903,7 @@ class YellowText(QGraphicsRectItem):
         else:
             self.moveBy(xOffset, yOffset)
 
-    def paint(self, painter, dummyOption, dummyWidget):
+    def paint(self, painter, unusedOption, unusedWidget):
         """override predefined paint"""
         painter.setFont(self.font)
         painter.fillRect(self.rect(), QBrush(QColor('yellow')))
@@ -959,7 +944,7 @@ class DiscardBoard(CourtBoard):
         uiTile.dark = False
         uiTile.focusable = False
         self.focusTile = uiTile
-        self.hasFocus = True
+        self.hasLogicalFocus = True
         self.lastDiscarded = uiTile
 
     def dropEvent(self, event):
