@@ -3,19 +3,8 @@
 """
 Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from qt import Qt, QAbstractTableModel, QModelIndex, QSize
@@ -34,7 +23,7 @@ class ChatModel(QAbstractTableModel):
     """a model for the chat view"""
 
     def __init__(self, parent=None):
-        super(ChatModel, self).__init__(parent)
+        super().__init__(parent)
         self.chatLines = []
 
     def headerData(self, section, orientation, role=Qt.DisplayRole):  # pylint: disable=no-self-use
@@ -43,12 +32,11 @@ class ChatModel(QAbstractTableModel):
             if orientation == Qt.Horizontal:
                 if section == 1:
                     return int(Qt.AlignRight)
-                else:
-                    return int(Qt.AlignLeft)
+                return int(Qt.AlignLeft)
         if orientation != Qt.Horizontal:
             return int(section + 1)
         if role != Qt.DisplayRole:
-            return
+            return None
         result = ''
         if section < self.columnCount():
             result = [i18n('Time'), i18n('Player'), i18n('Message')][section]
@@ -57,11 +45,12 @@ class ChatModel(QAbstractTableModel):
     def rowCount(self, parent=None):
         """how many lines are in the model?"""
         if parent and parent.isValid():
+            # similar in tables.py
             # we have only top level items
             return 0
         return len(self.chatLines)
 
-    def columnCount(self, dummyParent=None):  # pylint: disable=no-self-use
+    def columnCount(self, unusedParent=None):  # pylint: disable=no-self-use
         """for now we only have time, who, message"""
         return 3
 
@@ -71,8 +60,7 @@ class ChatModel(QAbstractTableModel):
         if role == Qt.TextAlignmentRole:
             if index.column() == 1:
                 return int(Qt.AlignRight)
-            else:
-                return int(Qt.AlignLeft)
+            return int(Qt.AlignLeft)
         if index.isValid() and (0 <= index.row() < len(self.chatLines)):
             chatLine = self.chatLines[index.row()]
             if role == Qt.DisplayRole and index.column() == 0:
@@ -86,7 +74,7 @@ class ChatModel(QAbstractTableModel):
             elif role == Qt.DisplayRole and index.column() == 2:
                 result = i18n(chatLine.message)
             elif role == Qt.ForegroundRole and index.column() == 2:
-                palette = Internal.app.palette() # pylint: disable=no-member
+                palette = Internal.app.palette()  # pylint: disable=no-member
                 color = 'blue' if chatLine.isStatusMessage else palette.windowText(
                 )
                 result = QColor(color)
@@ -125,7 +113,7 @@ class ChatWindow(QWidget):
     """a widget for showing chat messages"""
 
     def __init__(self, scene=None, table=None):
-        super(ChatWindow, self).__init__(None)
+        super().__init__(None)
         self.scene = scene
         self.table = table or scene.game.client.table
         self.table.chatWindow = self
